#include <qapplication.h>
#include "ControlBoard.h"
#include "Effects.h"

ControlBoard::ControlBoard( int banks, QWidget *parent, const char *name ) :
	QWidget( parent, name ),
	timer( new QTimer( this ) ),
	_banks( banks ),
	tempo( new QSlider( 20, 500, 5, 450, QSlider::Horizontal, this, "Speed" ) ),
	grid( new QGridLayout( this, 1, 1, 4, -1, NULL ) ),
	quit_button( new QPushButton( "Quit", this ) ),
	run( new QPushButton( "Run", this ) ),
	pause( new QPushButton( "Pause", this ) ),
	lb( new QListBox( this ) ),
	monitor( new DammitMonitor( banks ) )
{
	tempoChanged( 450 );
	grid->addMultiCellWidget( lb, 0, 1, 0, 2, 0 );
	grid->addMultiCellWidget( tempo, 2, 2, 0, 2, 0 );
	grid->addMultiCellWidget( run, 3, 3, 0, 0, 0 );
	grid->addMultiCellWidget( pause, 3, 3, 2, 2, 0 );
	grid->addMultiCellWidget( quit_button, 5, 5, 0, 2, 0 );

	grid->addColSpacing( 0, 20 );
	grid->setColStretch( 0, 1 );
	grid->setColStretch( 1, 1 );
	grid->setColStretch( 2, 1 );
	
	grid->addRowSpacing( 0, 100 );
	grid->addRowSpacing( 3, 4 );
	grid->setRowStretch( 0, 1 );		

	initEffects();
	
	monitor->setCaption( "Monitor" );
	monitor->show();

	pause->setEnabled( FALSE );

	QObject::connect( quit_button, SIGNAL(clicked()), qApp, SLOT( quit() ) );
        QObject::connect( timer, SIGNAL(timeout()), this, SLOT( timeout() ) );
       	QObject::connect( run, SIGNAL(clicked()), this, SLOT( runClicked() ) );
	QObject::connect( pause, SIGNAL(clicked()), this, SLOT( pauseClicked() ) );
	QObject::connect( tempo, SIGNAL( valueChanged(int) ), this, SLOT( tempoChanged(int) ) );
}

ControlBoard::~ControlBoard() {
	for( int i = 0; i < effects.size(); i++ ) {
		delete effects[i];
	}
	timer->stop();
}

void ControlBoard::runClicked() {
	en = lb->currentItem();
	if( en != -1 ) {
		effects[en]->reset();
		timer->start( speed, FALSE );
		pause->setEnabled( TRUE );
		pause->setCaption( "Pause" );
	}
}

void ControlBoard::pauseClicked() {
	if( timer->isActive() ) {
		timer->stop();
		pause->setText( "Resume" );
	}
	else {
		timer->start( speed, FALSE );
		pause->setText( "Pause" );
	}
}

void ControlBoard::timeout() {
	monitor->setFrame( effects[en]->getNextFrame() );
}

void ControlBoard::tempoChanged( int x ) {
	speed = 500 - x;
	if( timer->isActive() ) {
		timer->changeInterval( speed );
	}
}

void ControlBoard::initEffects() {
	effects.clear();
	effects.push_back( new PongEffect( _banks ) );
	effects.push_back( new ChaseEffect( _banks ) );
	effects.push_back( new WaterfallEffect( _banks ) );
	effects.push_back( new StrobeEffect( _banks ) );
	for( int i = 0; i < effects.size(); i++ ) {
		lb->insertItem( effects[i]->getName() );
	}
}

#include "mControlBoard.cpp"
