#include "Monitor.h"
#include <qpainter.h>
#include <qpixmap.h>
#include "dammit.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <iostream.h>
DammitMonitor::DammitMonitor( int banks, QWidget *parent, const char *name ) :
	QWidget( parent, name ),
#ifdef USEHW
	dammitfd( 0 ),
#endif
	_banks( setBanks( banks ) )
{
#ifdef USEHW
	// ASSERT( dammitfd >= 0 );
#endif
	setBackgroundMode( NoBackground );
}

DammitMonitor::~DammitMonitor() {
#ifdef USEHW
	close( dammitfd );
#endif
}

void DammitMonitor::setFrame( const DammitFrame &frame ) {
	_frame = frame;
	repaint();

#ifdef USEHW
	write( dammitfd, _frame(), _banks );
#endif
	emit frameChanged( _frame );
}

int DammitMonitor::setBanks( int banks ) {
	if( banks < 1 ) {
		banks = 1;
	}
	else if( banks > 8 ) {
		banks = 8;
	}
	
	_banks = banks;
	repaint();

#ifdef USEHW
	if( dammitfd != 0 ) {
		close( dammitfd );
	}

	if( _banks == 1 ) {
		dammitfd = open( "/dev/dammit0", O_WRONLY );
	}
	else {
		dammitfd = open( "/dev/dammit2", O_WRONLY );
		ioctl( dammitfd, DAMMIT_SET_MULTI, _banks );
	}
#endif

	emit banksChanged( banks );
	return _banks;
}

void DammitMonitor::paintEvent( QPaintEvent * ) {
	QRect	cr( rect()  );
	QPixmap  pix( cr.size() );
	QPainter q( this );
	QPainter p( &pix );

	pix.fill( black );

	p.setPen( NoPen );

	for( int x = 0; x < 8;  x++ ) {
		for( int y = 0; y < _banks; y++ ) {
			p.setBrush( (_frame[y] & ( 1 << x ) )?blue:white );

			p.drawPie( x*30+2, y*30+2, 26, 26, 0, 16*360 + 2 );
		}
	}
	bitBlt( this, 0, 0, &pix );
}

QSizePolicy DammitMonitor::sizePolicy() const {
	return QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
}

QSize DammitMonitor::sizeHint() const {
	return QSize( 8 * 30, _banks * 30 );
}

#include "mMonitor.cpp"
